@@ -1,6 +1,6 @@
 /*
  *  Rsrv.h : constants and macros for Rserve client/server architecture
- *  Copyright (C) 2002-13 Simon Urbanek
+ *  Copyright (C) 2002-21 Simon Urbanek
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published
@@ -32,7 +32,7 @@
 #include "config.h"
 #endif
 
-#define RSRV_VER 0x010702 /* Rserve v1.7-2 */
+#define RSRV_VER 0x010809 /* Rserve v1.8-9 */
 
 #define default_Rsrv_port 6311
 
@@ -77,10 +77,10 @@
 
  */
 
-struct phdr { /* always 16 bytes */
-	int cmd; /* command */
-	int len; /* length of the packet minus header (ergo -16) */
-	int dof; /* data offset behind header (ergo usually 0) */
+struct phdr {   /* always 16 bytes */
+	int cmd;    /* command */
+	int len;    /* length of the packet minus header (ergo -16) */
+	int msg_id; /* message id (since 1.8) [WAS:data offset behind header (ergo usually 0)] */
 	int res; /* high 32-bit of the packet length (since 0103
 				and supported on 64-bit platforms only)
 				aka "lenhi", but the name was not changed to
@@ -126,6 +126,8 @@ struct phdr { /* always 16 bytes */
 
 /* flag for create_server: Use QAP object-cap mode */
 #define SRV_QAP_OC 0x40
+/* mask of all flags that are relevant to QAP (so they can be passed through) */
+#define SRV_QAP_FLAGS (SRV_QAP_OC)
 
 /* stat codes; 0-0x3f are reserved for program specific codes - e.g. for R
    connection they correspond to the stat of Parse command.
@@ -346,6 +348,18 @@ struct phdr { /* always 16 bytes */
 #define ALIGN_DOUBLES
 #endif
 
+#ifndef SIZEOF_SIZE_T
+#include <Rconfig.h> /* defines SIZEOF_SIZE_T in case we missed it */
+#endif
+
+/* long vectors - we don't want to mandate Rinternals.h here
+   so we use the minimal definition */
+#if ( SIZEOF_SIZE_T > 4 )
+#include <stddef.h> /* for ptrdiff_t, which is required by C99 */
+typedef ptrdiff_t rlen_t;
+/* this is used for alignment/masking */
+#define rlen_max ((rlen_t) 0x7fffffffffffffff)
+#else /* old legacy definition using unsigned long */
 /* this is the type used to calculate pointer distances */
 /* note: we may want to use size_t or something more compatible */
 typedef unsigned long rlen_t;
@@ -359,7 +373,7 @@ typedef unsigned long rlen_t;
 #define rlen_max 0xffffffffL
 #endif /* __LP64__ */
 #endif /* ULONG_MAX */
-
+#endif /* long vector fallback */
 
 /* functions/macros to convert native endianess of int/double for transport
    currently ony PPC style and Intel style are supported */
